-- {{{ Start off clean. Drop all relations.
DROP VIEW IF EXISTS "systems_groups_from_hosts_types"       CASCADE;
DROP VIEW IF EXISTS "systems_groups_from_hosts"             CASCADE;
DROP VIEW IF EXISTS "systems_groups_all"                    CASCADE;
DROP VIEW IF EXISTS "systems_groups_any"                    CASCADE;
DROP VIEW IF EXISTS "systems_groups"                        CASCADE;
DROP VIEW IF EXISTS "systems_users_from_hosts_types"        CASCADE;
DROP VIEW IF EXISTS "systems_users_from_hosts"              CASCADE;
DROP VIEW IF EXISTS "systems_users_all"                     CASCADE;
DROP VIEW IF EXISTS "systems_users_any"                     CASCADE;
DROP VIEW IF EXISTS "systems_users_aggregate"               CASCADE;
DROP VIEW IF EXISTS "systems_users"                         CASCADE;
DROP VIEW IF EXISTS "systems_users_groups_from_hosts_types" CASCADE;
DROP VIEW IF EXISTS "systems_users_groups_from_hosts"       CASCADE;
DROP VIEW IF EXISTS "systems_users_groups_all"              CASCADE;
DROP VIEW IF EXISTS "systems_users_groups_any"              CASCADE;
DROP VIEW IF EXISTS "systems_users_groups"                  CASCADE;
DROP VIEW IF EXISTS "shell_overrides_from_hosts"            CASCADE;
DROP VIEW IF EXISTS "shell_overrides_any"                   CASCADE;
DROP VIEW IF EXISTS "user_sanctions_most_recent_per_user"   CASCADE;

DROP TABLE IF EXISTS "groups"         CASCADE;
DROP TABLE IF EXISTS "users"          CASCADE;
DROP TABLE IF EXISTS "roles"          CASCADE;
DROP TABLE IF EXISTS "hosts"          CASCADE;
DROP TABLE IF EXISTS "types"          CASCADE;
DROP TABLE IF EXISTS "entities"       CASCADE;
DROP TABLE IF EXISTS "users_groups"   CASCADE;
DROP TABLE IF EXISTS "roles_entities" CASCADE;
DROP TABLE IF EXISTS "types_roles"    CASCADE;
DROP TABLE IF EXISTS "hosts_types"    CASCADE;
DROP TABLE IF EXISTS "hosts_entities" CASCADE;
DROP TABLE IF EXISTS "misc_data"      CASCADE;
DROP TABLE IF EXISTS "user_sanctions" CASCADE;
-- }}}

-- Use this table as a generic key/value store
CREATE TABLE "misc_data" (
    "key"   TEXT PRIMARY KEY,
    "value" TEXT
);
INSERT INTO "misc_data" ("key", "value") VALUES ('default_user_shell', '/bin/bash');

CREATE TABLE "groups" (
    "id"          INTEGER               PRIMARY KEY,
    "groupname"   CHARACTER VARYING(16) NOT NULL,
    "description" TEXT,
    "password"    CHARACTER VARYING(20)
);

CREATE TABLE "users" (
    "username"                                  CHARACTER VARYING(64)  PRIMARY KEY,
    "password"                                  CHARACTER VARYING(128) NOT NULL,
    "id"                                        INTEGER                UNIQUE,  -- Allow NULL due to demographics data that doesn't have UNIX IDs.
    "group_id"                                  INTEGER,                        -- Allow NULL due to demographics data that doesn't have UNIX GIDs.
    "comment"                                   CHARACTER VARYING(128),
    "home_directory"                            CHARACTER VARYING(256),
    "shell"                                     TEXT,
    "encrypted_password"                        CHARACTER VARYING(128) NOT NULL,
    "date_of_last_password_change"              INTEGER                NOT NULL,
    "minimum_password_age"                      INTEGER                NOT NULL,
    "maximum_password_age"                      INTEGER                NOT NULL,
    "password_warning_period"                   INTEGER                NOT NULL,
    "password_inactivity_period"                INTEGER                NOT NULL,
    "account_expiration_date"                   INTEGER                NOT NULL,
    "reserved_field"                            INTEGER                NOT NULL,
    "legacy_home_directory"                     CHARACTER VARYING(256),
    "uf_home_directory"                         CHARACTER VARYING(256),
    -- demographics follows
    "freeform_name"                             TEXT,
    "address_work_postal"                       TEXT,
    "address_home_postal"                       TEXT,
    "address_work_office"                       TEXT,
    "address_peoplesoft_home"                   TEXT,
    "address_peoplesoft_mailing"                TEXT,
    "university_id"                             TEXT,
    "surname"                                   TEXT,
    "given_name"                                TEXT,
    "middle_initial"                            TEXT,
    "gender"                                    TEXT,
    "phone_number_home"                         TEXT,
    "phone_number_mobile"                       TEXT,
    "phone_number_work"                         TEXT,
    "phone_number_work_2nd"                     TEXT,
    "phone_number_dorm"                         TEXT,
    "birth_date"                                DATE,
    "academic_class"                            TEXT,
    "admissions_data"                           TEXT,
    "number_of_suppressed_fields"               INTEGER,
    "address_dorm"                              TEXT,
    "email_address_displayed"                   TEXT,
    "email_address_preferred_rfc_822_recipient" TEXT,
    CONSTRAINT "users__group_id_foreign_key" FOREIGN KEY ("group_id") REFERENCES "groups"("id")
);
CREATE INDEX "university_id_key" ON "users" ("university_id");

CREATE TABLE "suppressed_demographics" (
    "university_id"    TEXT NOT NULL,
    "suppressed_field" TEXT NOT NULL,
    "inserted_at"      TIMESTAMP WITH TIME ZONE NOT NULL DEFAULT now(),

    UNIQUE (
        "university_id",
        "suppressed_field"
    )

    -- TODO we cannot make use of the foreign key until we have unique university_id values in the users table.
    -- SELECT COUNT(*), university_id FROM users WHERE university_id IS NOT NULL GROUP BY university_id HAVING COUNT(*) > 1;

    -- CONSTRAINT "users__university_id__foreign_key" FOREIGN KEY ("university_id") REFERENCES "users"("university_id")
);

CREATE TABLE "appointments" (
    "university_id"              TEXT NOT NULL,
    "peoplesoft_department_code" TEXT NOT NULL,
    "apso_department_code"       TEXT NOT NULL,
    "peoplesoft_pay_code"        TEXT NOT NULL,
    "appointment_status"         TEXT NOT NULL,
    "appointment_time"           TEXT NOT NULL,
    "employee_type"              TEXT NOT NULL,
    "job_duration"               TEXT NOT NULL,
    "location"                   TEXT NOT NULL,
    "hours_per_week"             TEXT NOT NULL,
    "appointment_type_code"      TEXT NOT NULL,
    "department_description"     TEXT NOT NULL,
    "job_title"                  TEXT NOT NULL,
    "efs_dept_id_code"           TEXT NOT NULL,
    "rrc"                        TEXT NOT NULL,
    "primary_appointment_flag"   TEXT NOT NULL,
    "zdept_department_code"      TEXT NOT NULL,
    "verbatim"                   TEXT NOT NULL,
    "inserted_at"                TIMESTAMP WITH TIME ZONE NOT NULL DEFAULT now(),

    UNIQUE (
        "university_id",
        "peoplesoft_department_code",
        "apso_department_code",
        "peoplesoft_pay_code",
        "appointment_status",
        "appointment_time",
        "employee_type",
        "job_duration",
        "location",
        "hours_per_week",
        "appointment_type_code",
        "department_description",
        "job_title",
        "efs_dept_id_code",
        "rrc",
        "primary_appointment_flag",
        "zdept_department_code",
        "verbatim"
    )
);

CREATE TABLE "passwords" (
    "username"   CHARACTER VARYING(64) NOT NULL,
    "type"       TEXT                  NOT NULL,
    "password"   TEXT                  NOT NULL,
    "changed_at" TIMESTAMP WITH TIME ZONE,
    UNIQUE ("username", "type")
);

CREATE SEQUENCE "roles_id_seq" NO CYCLE;
CREATE TABLE "roles" (
    "id"             INTEGER PRIMARY KEY DEFAULT nextval('roles_id_seq'),
    "role"           TEXT    NOT NULL UNIQUE,
    "radius_visible" BOOLEAN
);
ALTER SEQUENCE "roles_id_seq" OWNED BY "roles"."id";

CREATE SEQUENCE "hosts_id_seq" NO CYCLE;
CREATE TABLE "hosts" (
    "id"            INTEGER               PRIMARY KEY DEFAULT nextval('hosts_id_seq'),

    -- See http://tools.ietf.org/html/rfc1035: "Labels must be 63 characters or less."
    "hostname"      CHARACTER VARYING(63) NOT NULL UNIQUE,

    "shell"         TEXT,  -- Leave NULL unless you want to restrict users' access.

    -- FreeRADIUS limits this to 31 characters. I'm not sure what the RFC
    --  limits are, but let's not impose any limits here.
    "radius_secret" TEXT,

    -- Puppet environment for ENC
    "environment"   TEXT
);
ALTER SEQUENCE "hosts_id_seq" OWNED BY "hosts"."id";

-- Enforce lower case UNIQUEness.
-- See http://stackoverflow.com/questions/9046825/postgresql-auto-lowercase-text-while-or-before-inserting-to-a-column
CREATE UNIQUE INDEX idx_lower_unique_hostname ON hosts (lower(hostname));

CREATE SEQUENCE "types_id_seq" NO CYCLE;
CREATE TABLE "types" (
    "id"             INTEGER PRIMARY KEY DEFAULT nextval('types_id_seq'),
    "type"           TEXT    NOT NULL UNIQUE,
    "radius_visible" BOOLEAN
);
ALTER SEQUENCE "types_id_seq" OWNED BY "types"."id";

-- Puppet classes for ENC
CREATE SEQUENCE "classes_id_seq" NO CYCLE;
CREATE TABLE "classes" (
    "id"    INTEGER PRIMARY KEY DEFAULT nextval('classes_id_seq'),
    "class" TEXT    NOT NULL UNIQUE
);
ALTER SEQUENCE "classes_id_seq" OWNED BY "classes"."id";

-- Puppet parameters for ENC
CREATE TABLE "parameters" (
    "host_id"   INTEGER NOT NULL,
    "parameter" TEXT    NOT NULL,
    "value"     TEXT,

    PRIMARY KEY ("host_id", "parameter"),
    CONSTRAINT "parameters__host_id_foreign_key" FOREIGN KEY ("host_id") REFERENCES "hosts"("id")
);

CREATE SEQUENCE "entities_id_seq" NO CYCLE;
CREATE TABLE "entities" (
    "id"     INTEGER PRIMARY KEY DEFAULT nextval('entities_id_seq'),
    "entity" TEXT    NOT NULL UNIQUE
);
ALTER SEQUENCE "entities_id_seq" OWNED BY "entities"."id";

CREATE SEQUENCE "users_groups_id_seq" NO CYCLE;
CREATE TABLE "users_groups" (
    "id"       INTEGER PRIMARY KEY DEFAULT nextval('users_groups_id_seq'),
    "user_id"  INTEGER NOT NULL,
    "group_id" INTEGER NOT NULL,
    UNIQUE ("user_id", "group_id"),
    CONSTRAINT "users_groups__user_id_foreign_key"  FOREIGN KEY ("user_id")  REFERENCES "users"("id"),
    CONSTRAINT "users_groups__group_id_foreign_key" FOREIGN KEY ("group_id") REFERENCES "groups"("id")
);
ALTER SEQUENCE "users_groups_id_seq" OWNED BY "users_groups"."id";

CREATE TABLE "roles_entities" (
    "role_id"          INTEGER NOT NULL,
    "entity_id"        INTEGER NOT NULL,
    "entity_record_id" INTEGER NOT NULL,

    -- priority can be used to order the results set from this table. Its impetus was
    -- due to the FreeRADIUS user_group view.
    "priority"         INTEGER,
    PRIMARY KEY ("role_id", "entity_id", "entity_record_id"),
    CONSTRAINT "roles_entities__role_id_foreign_key"   FOREIGN KEY ("role_id")   REFERENCES "roles"("id"),
    CONSTRAINT "roles_entities__entity_id_foreign_key" FOREIGN KEY ("entity_id") REFERENCES "entities"("id")
    -- NOTE
    -- There is no constraint for the entity_record_id. It needs to map to (currently) four different entities (three of which are tables).
    -- The three tables are users, groups, and users_groups. The fourth entity is user_shell_override.
);

CREATE VIEW "roles_users" AS
    SELECT
        "e"."entity",
        "u"."username" AS "entity_value",
        "r"."role",
        "t"."type",
        "h"."hostname"
    FROM
        "users"          "u"
        INNER JOIN
        "roles_entities" "r_e" ON "u"."id"        = "r_e"."entity_record_id"
        INNER JOIN
        "roles"          "r"   ON "r"."id"        = "r_e"."role_id"
        INNER JOIN
        "entities"       "e"   ON "e"."id"        = "r_e"."entity_id"
        LEFT JOIN
        "types_roles"    "t_r" ON "t_r"."role_id" = "r_e"."role_id"
        LEFT JOIN
        "types"          "t"   ON "t"."id"        = "t_r"."type_id"
        LEFT JOIN
        "hosts_types"    "h_t" ON "t_r"."type_id" = "h_t"."type_id"
        LEFT JOIN
        "hosts"          "h"   ON "h"."id"        = "h_t"."host_id"
    WHERE
        "e"."entity"    = 'users'
;

CREATE VIEW "roles_users_groups" AS
    SELECT
        "e"."entity",
        "u"."username"||' in '||"g"."groupname" AS "entity value",
        "r"."role",
        "t"."type",
        "h"."hostname"
    FROM
        "users"          "u"
        INNER JOIN
        "users_groups"   "u_g" ON "u"."id"        = "u_g"."user_id"
        INNER JOIN
        "groups"         "g"   ON "g"."id"        = "u_g"."group_id"
        INNER JOIN
        "roles_entities" "r_e" ON "u_g"."id"      = "r_e"."entity_record_id"
        INNER JOIN
        "roles"          "r"   ON "r"."id"        = "r_e"."role_id"
        INNER JOIN
        "entities"       "e"   ON "e"."id"        = "r_e"."entity_id"
        LEFT JOIN
        "types_roles"    "t_r" ON "t_r"."role_id" = "r_e"."role_id"
        LEFT JOIN
        "types"          "t"   ON "t"."id"        = "t_r"."type_id"
        LEFT JOIN
        "hosts_types"    "h_t" ON "t_r"."type_id" = "h_t"."type_id"
        LEFT JOIN
        "hosts"          "h"   ON "h"."id"        = "h_t"."host_id"
    WHERE
        "e"."entity"    = 'users_groups'
;

CREATE VIEW "roles_full" AS
    SELECT * FROM "roles_users"
    UNION
    SELECT * FROM "roles_users_groups"
;

CREATE TABLE "types_roles" (
    "type_id" INTEGER NOT NULL,
    "role_id" INTEGER NOT NULL,
    PRIMARY KEY ("type_id", "role_id"),
    CONSTRAINT "types_roles__type_id_foreign_key" FOREIGN KEY ("type_id") REFERENCES "types"("id"),
    CONSTRAINT "types_roles__role_id_foreign_key" FOREIGN KEY ("role_id") REFERENCES "roles"("id")
);

CREATE TABLE "hosts_types" (
    "host_id"  INTEGER NOT NULL,
    "type_id"  INTEGER NOT NULL,

    -- See comment about priority in roles_entities.
    "priority" INTEGER,
    PRIMARY KEY ("host_id", "type_id"),
    CONSTRAINT "hosts_types__host_id_foreign_key" FOREIGN KEY ("host_id") REFERENCES "hosts"("id"),
    CONSTRAINT "hosts_types__type_id_foreign_key" FOREIGN KEY ("type_id") REFERENCES "types"("id")
);

CREATE TABLE "hosts_classes" (
    "host_id"  INTEGER NOT NULL,
    "class_id" INTEGER NOT NULL,

    PRIMARY KEY ("host_id", "class_id"),
    CONSTRAINT "hosts_classes__host_id_foreign_key"  FOREIGN KEY ("host_id")  REFERENCES "hosts"("id"),
    CONSTRAINT "hosts_classes__class_id_foreign_key" FOREIGN KEY ("class_id") REFERENCES "classes"("id")
);

CREATE TABLE "hosts_parameters" (
    "host_id"      INTEGER NOT NULL,
    "parameter_id" INTEGER NOT NULL,

    PRIMARY KEY ("host_id", "parameter_id"),
    CONSTRAINT "hosts_parameters__host_id_foreign_key"      FOREIGN KEY ("host_id")      REFERENCES "hosts"("id"),
    CONSTRAINT "hosts_parameters__parameter_id_foreign_key" FOREIGN KEY ("parameter_id") REFERENCES "parameters"("id")
);

CREATE TABLE "hosts_entities" (
    "host_id"          INTEGER NOT NULL,
    "entity_id"        INTEGER NOT NULL,
    "entity_record_id" INTEGER NOT NULL,
    PRIMARY KEY ("host_id", "entity_id", "entity_record_id"),
    CONSTRAINT "hosts_entities__host_id_foreign_key"   FOREIGN KEY ("host_id")   REFERENCES "hosts"("id"),
    CONSTRAINT "hosts_entities__entity_id_foreign_key" FOREIGN KEY ("entity_id") REFERENCES "entities"("id")
    -- NOTE
    -- There is no constraint for the entity_record_id. It needs to map to (currently) three different tables.
    -- The three tables are users, groups, and users_groups.
);

CREATE SEQUENCE "user_sanctions_id_seq" NO CYCLE;
CREATE TABLE "user_sanctions" (
    "id"          INTEGER PRIMARY KEY DEFAULT nextval('user_sanctions_id_seq'),
    "user_id"     INTEGER                  NOT NULL,
    "description" TEXT                     NOT NULL,
    "shall_deny"  BOOLEAN                  NOT NULL,
    "occurred_at" TIMESTAMP WITH TIME ZONE NOT NULL,
    "resolved_at" TIMESTAMP WITH TIME ZONE,
    "misc_data"   TEXT,
    CONSTRAINT "user_sanctions__user_id_foreign_key" FOREIGN KEY ("user_id") REFERENCES "users"("id")
);

CREATE VIEW "user_sanctions_most_recent_per_user" AS
    SELECT
        "u_s"."user_id"          AS "user_id",
        "u_s"."description"      AS "description",
        "u_s"."occurred_at"      AS "occurred_at"
    FROM
        "user_sanctions" "u_s",
        (
            SELECT
                "u_s_subquery"."user_id"          AS "user_id",
                MAX("u_s_subquery"."occurred_at") AS "occurred_at"
            FROM
                "user_sanctions" "u_s_subquery"
            WHERE
                "u_s_subquery"."shall_deny" = 't'
            GROUP BY
                "u_s_subquery"."user_id"
        ) "u_s_most_recent_sanction"
    WHERE
        "u_s"."user_id"     = "u_s_most_recent_sanction"."user_id"     AND
        "u_s"."occurred_at" = "u_s_most_recent_sanction"."occurred_at"
;

-- {{{ Helper VIEWS for dealing with groups
CREATE VIEW "systems_groups_from_hosts_types" AS
    SELECT
        "g".*,
        "h"."hostname",
        "h"."id" AS "host_id"
    FROM
        "groups"         "g",
        "hosts"          "h",
        "entities"       "e",
        "roles_entities" "r_e",
        "types_roles"    "t_r",
        "hosts_types"    "h_t"
    WHERE
        "e"."entity"    = 'groups'                 AND
        "e"."id"        = "r_e"."entity_id"        AND
        "g"."id"        = "r_e"."entity_record_id" AND
        "r_e"."role_id" = "t_r"."role_id"          AND
        "t_r"."type_id" = "h_t"."type_id"          AND
        "h_t"."host_id" = "h"."id";

CREATE VIEW "systems_groups_from_hosts" AS
    SELECT
        "g".*,
        "h"."hostname",
        "h"."id" AS "host_id"
    FROM
        "groups"         "g",
        "hosts"          "h",
        "entities"       "e",
        "hosts_entities" "h_e"
    WHERE
        "e"."entity"    = 'groups'                 AND
        "e"."id"        = "h_e"."entity_id"        AND
        "g"."id"        = "h_e"."entity_record_id" AND
        "h_e"."host_id" = "h"."id";

CREATE VIEW "systems_groups_all" AS
    SELECT
        "g".*,
        "h"."hostname",
        "h"."id" AS "host_id"
    FROM
        "hosts" "h"
        INNER JOIN
        "hosts_types" "h_t" ON "h"."id" = "h_t"."host_id"
        INNER JOIN
        "types" "t" ON "t"."id" = "h_t"."type_id"
        CROSS JOIN
        "groups" "g"
    WHERE
        "t"."type" = 'ALL:groups';

CREATE VIEW "systems_groups_any" AS
    SELECT
        "g".*,
        -- Use NULL in the following query result set in the
        -- hostname field. Use 0 as the hostid for users that
        -- should appear on any system.
        NULL AS "hostname",
        0 AS "host_id"
    FROM
        "groups"         "g",
        "roles"          "r",
        "entities"       "e",
        "roles_entities" "r_e"
    WHERE
        "e"."entity"    = 'groups'                 AND
        "e"."id"        = "r_e"."entity_id"        AND
        "g"."id"        = "r_e"."entity_record_id" AND
        "r_e"."role_id" = "r"."id"                 AND
        "r"."role"      = 'ANY';

CREATE VIEW "systems_groups" AS
    SELECT * FROM "systems_groups_from_hosts_types"
    UNION
    SELECT * FROM "systems_groups_from_hosts"
    UNION
    SELECT * FROM "systems_groups_all"
    UNION
    -- Use NULL in the following query result set in the
    -- hostname field.
    SELECT * FROM "systems_groups_any";
    -- When querying this view, use a 'hostname IS NULL'
    -- predicate to select groups that should exist on all
    -- systems.
-- }}}

-- {{{ Shell override helper views
CREATE VIEW "shell_overrides_from_hosts" AS
    SELECT
        "u"."id",
        COALESCE("u"."shell", "m_d__default_user_shell"."value") AS "shell_override",
        "h"."hostname",
        "h"."id" AS "host_id"
    FROM
        "misc_data"      "m_d__default_user_shell",
        "users"          "u",
        "hosts"          "h",
        "hosts_entities" "h_e",
        "entities"       "e"
    WHERE
        "e"."entity"                    = 'user_shell_override'    AND
        "e"."id"                        = "h_e"."entity_id"        AND
        "u"."id"                        = "h_e"."entity_record_id" AND
        "h"."id"                        = "h_e"."host_id"          AND
        "m_d__default_user_shell"."key" = 'default_user_shell';

CREATE VIEW "shell_overrides_any" AS
    SELECT
        "u"."id",
        COALESCE("u"."shell", "m_d__default_user_shell"."value") AS "shell_override",
        NULL AS "hostname",
        0 AS "host_id"
    FROM
        "misc_data"      "m_d__default_user_shell",
        "users"          "u",
        "roles"          "r",
        "entities"       "e",
        "roles_entities" "r_e"
    WHERE
        "e"."entity"                    = 'users'                  AND
        "e"."id"                        = "r_e"."entity_id"        AND
        "u"."id"                        = "r_e"."entity_record_id" AND
        "r_e"."role_id"                 = "r"."id"                 AND
        "r"."role"                      = 'ANY'                    AND
        "m_d__default_user_shell"."key" = 'default_user_shell';
-- }}}

-- {{{ Helper VIEWS for dealing with users
CREATE VIEW "systems_users_from_hosts_types" AS
    SELECT
        "u".*,
        COALESCE("s_o_a"."shell_override", "s_o_h"."shell_override", "h"."shell", "u"."shell", "m_d__default_user_shell"."value") AS "host_shell",
        "h"."hostname",
        "h"."id" AS "host_id"
    FROM
        "misc_data"                  "m_d__default_user_shell",
        "hosts"                      "h"
        INNER JOIN
        "hosts_types"                "h_t"   ON "h"."id"        = "h_t"."host_id"
        INNER JOIN
        "types_roles"                "t_r"   ON "t_r"."type_id" = "h_t"."type_id"
        INNER JOIN
        "roles_entities"             "r_e"   ON "r_e"."role_id" = "t_r"."role_id"
        INNER JOIN
        "entities"                   "e"     ON "e"."id"        = "r_e"."entity_id"
        INNER JOIN
        "users"                      "u"     ON "u"."id"        = "r_e"."entity_record_id"
        LEFT JOIN
        "shell_overrides_any"        "s_o_a" ON "s_o_a"."id"    = "u"."id"
        LEFT JOIN
        "shell_overrides_from_hosts" "s_o_h" ON "s_o_h"."id"    = "u"."id" AND "s_o_h"."host_id" = "h"."id"
    WHERE
        "e"."entity"                    = 'users'              AND
        "m_d__default_user_shell"."key" = 'default_user_shell'
;

CREATE VIEW "systems_users_from_hosts" AS
    SELECT
        "u".*,
        COALESCE("s_o_a"."shell_override", "s_o_h"."shell_override", "h"."shell", "u"."shell", "m_d__default_user_shell"."value") AS "host_shell",
        "h"."hostname",
        "h"."id" AS "host_id"
    FROM
        "misc_data"                  "m_d__default_user_shell",
        "hosts"                      "h"
        INNER JOIN
        "hosts_entities"             "h_e"   ON "h"."id"     = "h_e"."host_id"
        INNER JOIN
        "entities"                   "e"     ON "e"."id"     = "h_e"."entity_id"
        INNER JOIN
        "users"                      "u"     ON "u"."id"     = "h_e"."entity_record_id"
        LEFT JOIN
        "shell_overrides_any"        "s_o_a" ON "s_o_a"."id" = "u"."id"
        LEFT JOIN
        "shell_overrides_from_hosts" "s_o_h" ON "s_o_h"."id" = "u"."id" AND "s_o_h"."host_id" = "h"."id"
    WHERE
        "e"."entity"                    = 'users'              AND
        "m_d__default_user_shell"."key" = 'default_user_shell'
;

CREATE VIEW "systems_users_all" AS
    SELECT
        "u".*,
        COALESCE("s_o_a"."shell_override", "s_o_h"."shell_override", "h"."shell", "u"."shell", "m_d__default_user_shell"."value") AS "host_shell",
        "h"."hostname",
        "h"."id" AS "host_id"
    FROM
        "misc_data"                  "m_d__default_user_shell",
        "hosts"                      "h"
        INNER JOIN
        "hosts_types"                "h_t"   ON "h"."id"     = "h_t"."host_id"
        INNER JOIN
        "types"                      "t"     ON "t"."id"     = "h_t"."type_id"
        CROSS JOIN
        "users"                      "u"
        LEFT JOIN
        "shell_overrides_any"        "s_o_a" ON "s_o_a"."id" = "u"."id"
        LEFT JOIN
        "shell_overrides_from_hosts" "s_o_h" ON "s_o_h"."id" = "u"."id" AND "s_o_h"."host_id" = "h"."id"
    WHERE
        "t"."type"                      = 'ALL:users'          AND
        "u"."id"                        IS NOT NULL            AND
        "m_d__default_user_shell"."key" = 'default_user_shell'
;

CREATE VIEW "systems_users_any" AS
    SELECT
        "u".*,
        COALESCE("u"."shell", "m_d__default_user_shell"."value") AS "host_shell",
        -- Use NULL in the following query result set in the
        -- hostname field. Use 0 as the hostid for users that
        -- should appear on any system.
        NULL AS "hostname",
        0 AS "host_id"
    FROM
        "misc_data"      "m_d__default_user_shell",
        "users"          "u",
        "roles"          "r",
        "entities"       "e",
        "roles_entities" "r_e"
    WHERE
        "e"."entity"                    = 'users'                  AND
        "e"."id"                        = "r_e"."entity_id"        AND
        "u"."id"                        = "r_e"."entity_record_id" AND
        "r_e"."role_id"                 = "r"."id"                 AND
        "r"."role"                      = 'ANY'                    AND
        "m_d__default_user_shell"."key" = 'default_user_shell'
;

-- When querying this view, use a 'hostname IS NULL'
-- predicate to select users who should exist on all
-- systems.
CREATE VIEW "systems_users_aggregate" AS
    SELECT * FROM "systems_users_from_hosts_types"
    UNION
    SELECT * FROM "systems_users_from_hosts"
    UNION
    SELECT * FROM "systems_users_all"
    UNION
    SELECT * FROM "systems_users_any"
;

CREATE VIEW "systems_users" AS
    SELECT
        "s_u_a"."username",
        "s_u_a"."password",
        "s_u_a"."id",
        "s_u_a"."group_id",
        "s_u_a"."comment",
        "s_u_a"."home_directory",
        "s_u_a"."legacy_home_directory",
        "s_u_a"."uf_home_directory",
        "s_u_a"."host_shell" AS "shell",
        "s_u_a"."encrypted_password",
        "s_u_a"."date_of_last_password_change",
        "s_u_a"."minimum_password_age",
        "s_u_a"."maximum_password_age",
        "s_u_a"."password_warning_period",
        "s_u_a"."password_inactivity_period",
        "s_u_a"."account_expiration_date",
        "s_u_a"."reserved_field",
        "s_u_a"."hostname",
        "s_u_a"."host_id"
    FROM
        "systems_users_aggregate" "s_u_a"
;
-- }}}


-- {{{ Helper VIEWS for dealing with users_groups
CREATE VIEW "systems_users_groups_from_hosts_types" AS
    SELECT
        "u_g".*,
        "h"."hostname",
        "h"."id" AS "host_id"
    FROM
        "users_groups"   "u_g",
        "hosts"          "h",
        "entities"       "e",
        "roles_entities" "r_e",
        "types_roles"    "t_r",
        "hosts_types"    "h_t"
    WHERE
        "e"."entity"    = 'users_groups'           AND
        "e"."id"        = "r_e"."entity_id"        AND
        "u_g"."id"      = "r_e"."entity_record_id" AND
        "r_e"."role_id" = "t_r"."role_id"          AND
        "t_r"."type_id" = "h_t"."type_id"          AND
        "h_t"."host_id" = "h"."id"
;

CREATE VIEW "systems_users_groups_from_hosts" AS
    SELECT
        "u_g".*,
        "h"."hostname",
        "h"."id" AS "host_id"
    FROM
        "users_groups"   "u_g",
        "hosts"          "h",
        "entities"       "e",
        "hosts_entities" "h_e"
    WHERE
        "e"."entity"    = 'users_groups'           AND
        "e"."id"        = "h_e"."entity_id"        AND
        "u_g"."id"      = "h_e"."entity_record_id" AND
        "h_e"."host_id" = "h"."id"
;

CREATE VIEW "systems_users_groups_all" AS
    SELECT
        "u_g".*,
        "h"."hostname",
        "h"."id" AS "host_id"
    FROM
        "hosts" "h"
        INNER JOIN
        "hosts_types" "h_t" ON "h"."id" = "h_t"."host_id"
        INNER JOIN
        "types" "t" ON "t"."id" = "h_t"."type_id"
        CROSS JOIN
        "users_groups" "u_g"
    WHERE
        "t"."type" = 'ALL:users_groups'
;

CREATE VIEW "systems_users_groups_any" AS
    SELECT
        "u_g".*,
        -- Use NULL in the following query result set in the
        -- hostname field.
        NULL AS "hostname",
        0 AS "host_id"
    FROM
        "users_groups"   "u_g",
        "roles"          "r",
        "entities"       "e",
        "roles_entities" "r_e"
    WHERE
        "e"."entity"    = 'users_groups'           AND
        "e"."id"        = "r_e"."entity_id"        AND
        "u_g"."id"      = "r_e"."entity_record_id" AND
        "r_e"."role_id" = "r"."id"                 AND
        "r"."role"      = 'ANY'
;

CREATE VIEW "systems_users_groups" AS
    SELECT * FROM "systems_users_groups_from_hosts_types"
    UNION
    SELECT * FROM "systems_users_groups_from_hosts"
    UNION
    SELECT * FROM "systems_users_groups_all"
    UNION
    SELECT * FROM "systems_users_groups_any"
;
    -- When querying this view, use a 'hostname IS NULL'
    -- predicate to select users_groups that should exist on
    -- all systems.
-- }}}


-- {{{ Helper view for authorizing web access through Apache.
CREATE VIEW "web_users" AS
    SELECT
        "u"."id",
        "u"."username",
        "r"."role"
    FROM
        "users"          "u",
        "roles"          "r",
        "entities"       "e",
        "roles_entities" "r_e"
    WHERE
        "e"."entity"    = 'users'                  AND
        "e"."id"        = "r_e"."entity_id"        AND
        "u"."id"        = "r_e"."entity_record_id" AND
        "r_e"."role_id" = "r"."id"
;
-- }}}

-- {{{ RADIUS views

-- A question was asked on the FreeRADIUS mailing list:
-- Is the "id" field used by FR? Or does it exist for administration of
-- the table in case someone enters two records with identical (UserName,
-- Attribute, op, Values) values?

-- Answer:
-- Administration. Some tools (like pgAdmin) won't operate on a table unless there's a primary key.
-- Primary key could be UserName, Attribute, Op, Value, but there are some cases (using -=, or += for example) where two or more identical rows could exist.

-- Since this is just a view and the data is coming from a different table,
-- we don't need to be concerned with the id field.
CREATE VIEW "radius_check" AS
    SELECT
        0::INTEGER     AS "id",
        "p"."username" AS "username",
        "p"."type"     AS "attribute",
        ':='::TEXT     AS "op",
        "p"."password" AS "value"
    FROM
        "passwords" "p"
    WHERE
        "p"."type" = 'Crypt-Password'
;

CREATE VIEW "radius_nas" AS
    SELECT
        "h"."id"            AS "id",
        "h"."hostname"      AS "nasname",
        "h"."hostname"      AS "shortname",
        NULL                AS "type",
        "h"."radius_secret" AS "secret",
        NULL                AS "server"
    FROM
        "hosts" "h"
    WHERE
        "h"."radius_secret" IS NOT NULL
;

CREATE VIEW "radius_user_group" AS
    SELECT
        "u"."username"                AS "username",
        "r"."role"                    AS "groupname",
        COALESCE("r_e"."priority", 0) AS "priority"
    FROM
        "users"          "u",
        "roles"          "r",
        "entities"       "e",
        "roles_entities" "r_e"
    WHERE
        "e"."entity"         = 'users'                  AND
        "e"."id"             = "r_e"."entity_id"        AND
        "u"."id"             = "r_e"."entity_record_id" AND
        "r_e"."role_id"      = "r"."id"                 AND
        "r"."radius_visible" = 't'
;

CREATE VIEW "radius_nas_type" AS
    SELECT
        "h"."hostname" AS "hostname",
        "t"."type"     AS "type"
    FROM
        "types" "t",
        "hosts" "h",
        "hosts_types" "h_t"
    WHERE
        "h"."id"             = "h_t"."host_id" AND
        "t"."id"             = "h_t"."type_id" AND
        "t"."radius_visible" = 't'
    ORDER BY
        "h_t"."priority"
;

CREATE TABLE "radius_reply_content" (
    "user_id"   INTEGER      NOT NULL,
    "attribute" VARCHAR(64)  NOT NULL DEFAULT '',
    "op"        CHAR(2)      NOT NULL DEFAULT '=',
    "value"     VARCHAR(253) NOT NULL DEFAULT '',
    UNIQUE ("user_id", "attribute", "op", "value"),
    CONSTRAINT "radius_reply_content__user_id_foreign_key" FOREIGN KEY ("user_id") REFERENCES "users"("id")
);

-- add in the username to radius_reply_content
-- to get the resulting radius_reply view
CREATE VIEW "radius_reply" AS
    SELECT
        "r_r_c"."user_id"   AS "id",
        "u"."username"      AS "username",
        "r_r_c"."attribute" AS "attribute",
        "r_r_c"."op"        AS "op",
        "r_r_c"."value"     AS "value"
    FROM
        "radius_reply_content" "r_r_c",
        "users"                "u"
    WHERE
        "u"."id" = "r_r_c"."user_id"
;

CREATE TABLE "radius_group_check_content" (
    "role_id"   INTEGER      NOT NULL,
    "attribute" VARCHAR(64)  NOT NULL DEFAULT '',
    "op"        CHAR(2)      NOT NULL DEFAULT '==',
    "value"     VARCHAR(253) NOT NULL DEFAULT '',
    UNIQUE ("role_id", "attribute", "op", "value"),
    CONSTRAINT "radius_group_check_content__role_id_foreign_key" FOREIGN KEY ("role_id") REFERENCES "roles"("id")
);

CREATE VIEW "radius_group_check" AS
    SELECT
        "r_g_c_c"."role_id"   AS "id",
        "r"."role"            AS "groupname",
        "r_g_c_c"."attribute" AS "attribute",
        "r_g_c_c"."op"        AS "op",
        "r_g_c_c"."value"     AS "value"
    FROM
        "radius_group_check_content" "r_g_c_c",
        "roles"                      "r"
    WHERE
        "r"."id" = "r_g_c_c"."role_id"
;

CREATE TABLE "radius_group_reply_content" (
    "role_id"   INTEGER      NOT NULL,
    "attribute" VARCHAR(64)  NOT NULL DEFAULT '',
    "op"        CHAR(2)      NOT NULL DEFAULT '=',
    "value"     VARCHAR(253) NOT NULL DEFAULT '',
    UNIQUE ("role_id", "attribute", "op", "value"),
    CONSTRAINT "radius_group_reply_content__role_id_foreign_key" FOREIGN KEY ("role_id") REFERENCES "roles"("id")
);

CREATE VIEW "radius_group_reply" AS
    SELECT
        "r_g_r_c"."role_id"   AS "id",
        "r"."role"            AS "groupname",
        "r_g_r_c"."attribute" AS "attribute",
        "r_g_r_c"."op"        AS "op",
        "r_g_r_c"."value"     AS "value"
    FROM
        "radius_group_reply_content" "r_g_r_c",
        "roles"                      "r"
    WHERE
        "r"."id" = "r_g_r_c"."role_id"
;

-- }}}

-- {{{ VPN RADIUS views
DROP VIEW  IF EXISTS "vpn_radius_check_users"          CASCADE;
DROP VIEW  IF EXISTS "vpn_radius_check_user_sanctions" CASCADE;
DROP VIEW  IF EXISTS "vpn_radius_user_group"           CASCADE;
DROP TABLE IF EXISTS "vpn_radius_reply_content"        CASCADE;
DROP VIEW  IF EXISTS "vpn_radius_reply"                CASCADE;
DROP TABLE IF EXISTS "vpn_radius_group_check_content"  CASCADE;
DROP VIEW  IF EXISTS "vpn_radius_group_check"          CASCADE;
DROP VIEW  IF EXISTS "vpn_radius_group_check"          CASCADE;
DROP TABLE IF EXISTS "vpn_radius_group_reply_content"  CASCADE;
DROP VIEW  IF EXISTS "vpn_radius_group_reply"          CASCADE;

CREATE VIEW "vpn_radius_user_sanctions" AS
    SELECT
        "u".*,
        "u_s".*,
        COALESCE("m_d__prefix"."value", '') ||
        "u_s"."description"                 ||
        COALESCE("m_d__suffix"."value", '')    AS "reply_message"
    FROM
        "users"                               "u",
        "user_sanctions_most_recent_per_user" "u_s"
        LEFT JOIN
        "misc_data"                           "m_d__prefix" ON "m_d__prefix"."key" = 'vpn_static_reject_prefix'
        LEFT JOIN
        "misc_data"                           "m_d__suffix" ON "m_d__suffix"."key" = 'vpn_static_reject_suffix'
    WHERE
        "u"."id" = "u_s"."user_id"
;

CREATE VIEW "vpn_radius_user_group" AS
    SELECT
        "u"."username"                AS "username",
        "r"."role"                    AS "groupname",
        COALESCE("r_e"."priority", 0) AS "priority"
    FROM
        "users"          "u",
        "roles"          "r",
        "entities"       "e",
        "roles_entities" "r_e"
    WHERE
        "e"."entity"         = 'users'                  AND
        "e"."id"             = "r_e"."entity_id"        AND
        "u"."id"             = "r_e"."entity_record_id" AND
        "r_e"."role_id"      = "r"."id"                 AND
        "r"."radius_visible" = 't'
;

-- DELETE FROM misc_data WHERE key ~ '^vpn_static_reject_';
-- INSERT INTO misc_data (key, value) VALUES ('vpn_static_reject_prefix', 'Access to UMD VPN is denied:
-- ');
-- INSERT INTO misc_data (key, value) VALUES ('vpn_static_reject_suffix', '
-- Please go to https://rt.d.umn.edu/rt/SelfService/SIRT/ResolveSecurityIncidents.html to resolve your security issue.');

CREATE TABLE "vpn_radius_group_check_content" (
    "role_id"   INTEGER      NOT NULL,
    "attribute" VARCHAR(64)  NOT NULL DEFAULT '',
    "op"        CHAR(2)      NOT NULL DEFAULT '==',
    "value"     VARCHAR(253) NOT NULL DEFAULT '',
    UNIQUE ("role_id", "attribute", "op", "value"),
    CONSTRAINT "radius_group_check_content__role_id_foreign_key" FOREIGN KEY ("role_id") REFERENCES "roles"("id")
);

CREATE VIEW "vpn_radius_group_check" AS
    SELECT
        "r_g_c_c"."role_id"   AS "id",
        "r"."role"            AS "groupname",
        "r_g_c_c"."attribute" AS "attribute",
        "r_g_c_c"."op"        AS "op",
        "r_g_c_c"."value"     AS "value"
    FROM
        "radius_group_check_content" "r_g_c_c",
        "roles"                      "r"
    WHERE
        "r"."id" = "r_g_c_c"."role_id"
;

CREATE TABLE "vpn_radius_group_reply_content" (
    "role_id"   INTEGER      NOT NULL,
    "attribute" VARCHAR(64)  NOT NULL DEFAULT '',
    "op"        CHAR(2)      NOT NULL DEFAULT '=',
    "value"     VARCHAR(253) NOT NULL DEFAULT '',
    UNIQUE ("role_id", "attribute", "op", "value"),
    CONSTRAINT "radius_group_reply_content__role_id_foreign_key" FOREIGN KEY ("role_id") REFERENCES "roles"("id")
);

CREATE VIEW "vpn_radius_group_reply" AS
    SELECT
        "r_g_r_c"."role_id"   AS "id",
        "r"."role"            AS "groupname",
        "r_g_r_c"."attribute" AS "attribute",
        "r_g_r_c"."op"        AS "op",
        "r_g_r_c"."value"     AS "value"
    FROM
        "radius_group_reply_content" "r_g_r_c",
        "roles"                      "r"
    WHERE
        "r"."id" = "r_g_r_c"."role_id"
;

CREATE SEQUENCE "vpn_radius_profiles_id_seq" NO CYCLE;
CREATE TABLE "vpn_radius_profiles" (
    "id"          INTEGER PRIMARY KEY DEFAULT nextval('vpn_radius_profiles_id_seq'),
    "description" TEXT NOT NULL UNIQUE,
    "hex_string"  TEXT NOT NULL UNIQUE
);
-- }}}

-- {{{ PacketFence (PF) RADIUS views
DROP VIEW  IF EXISTS "packet_fence_radius_check" CASCADE;
CREATE VIEW "packet_fence_radius_check" AS
    SELECT
        0::INTEGER     AS "id",
        "p"."username" AS "username",
        "p"."type"     AS "attribute",
        ':='::TEXT     AS "op",
        "p"."password" AS "value"
    FROM
        "passwords" "p"
    WHERE
        "p"."type" = 'NT-Password'
;

CREATE VIEW "packet_fence_radius_user_group" AS
    SELECT
        "u"."username"                AS "username",
        "r"."role"                    AS "groupname",
        COALESCE("r_e"."priority", 0) AS "priority"
    FROM
        "users"          "u",
        "roles"          "r",
        "entities"       "e",
        "roles_entities" "r_e"
    WHERE
        "e"."entity"         = 'users'                  AND
        "e"."id"             = "r_e"."entity_id"        AND
        "u"."id"             = "r_e"."entity_record_id" AND
        "r_e"."role_id"      = "r"."id"                 AND
        "r"."radius_visible" = 't'
;

CREATE TABLE "packet_fence_radius_reply_content" (
    "user_id"   INTEGER      NOT NULL,
    "attribute" VARCHAR(64)  NOT NULL DEFAULT '',
    "op"        CHAR(2)      NOT NULL DEFAULT '=',
    "value"     VARCHAR(253) NOT NULL DEFAULT '',
    UNIQUE ("user_id", "attribute", "op", "value"),
    CONSTRAINT "packet_fence_radius_reply_content__user_id_foreign_key" FOREIGN KEY ("user_id") REFERENCES "users"("id")
);

-- add in the username to radius_reply_content
-- to get the resulting radius_reply view
CREATE VIEW "packet_fence_radius_reply" AS
    SELECT
        "r_r_c"."user_id"   AS "id",
        "u"."username"      AS "username",
        "r_r_c"."attribute" AS "attribute",
        "r_r_c"."op"        AS "op",
        "r_r_c"."value"     AS "value"
    FROM
        "packet_fence_radius_reply_content" "r_r_c",
        "users"                "u"
    WHERE
        "u"."id" = "r_r_c"."user_id"
;

CREATE TABLE "packet_fence_radius_group_check_content" (
    "role_id"   INTEGER      NOT NULL,
    "attribute" VARCHAR(64)  NOT NULL DEFAULT '',
    "op"        CHAR(2)      NOT NULL DEFAULT '==',
    "value"     VARCHAR(253) NOT NULL DEFAULT '',
    UNIQUE ("role_id", "attribute", "op", "value"),
    CONSTRAINT "packet_fence_radius_group_check_content__role_id_foreign_key" FOREIGN KEY ("role_id") REFERENCES "roles"("id")
);

CREATE VIEW "packet_fence_radius_group_check" AS
    SELECT
        "r_g_c_c"."role_id"   AS "id",
        "r"."role"            AS "groupname",
        "r_g_c_c"."attribute" AS "attribute",
        "r_g_c_c"."op"        AS "op",
        "r_g_c_c"."value"     AS "value"
    FROM
        "packet_fence_radius_group_check_content" "r_g_c_c",
        "roles"                      "r"
    WHERE
        "r"."id" = "r_g_c_c"."role_id"
;

CREATE TABLE "packet_fence_radius_group_reply_content" (
    "role_id"   INTEGER      NOT NULL,
    "attribute" VARCHAR(64)  NOT NULL DEFAULT '',
    "op"        CHAR(2)      NOT NULL DEFAULT '=',
    "value"     VARCHAR(253) NOT NULL DEFAULT '',
    UNIQUE ("role_id", "attribute", "op", "value"),
    CONSTRAINT "packet_fence_radius_group_reply_content__role_id_foreign_key" FOREIGN KEY ("role_id") REFERENCES "roles"("id")
);

CREATE VIEW "packet_fence_radius_group_reply" AS
    SELECT
        "r_g_r_c"."role_id"   AS "id",
        "r"."role"            AS "groupname",
        "r_g_r_c"."attribute" AS "attribute",
        "r_g_r_c"."op"        AS "op",
        "r_g_r_c"."value"     AS "value"
    FROM
        "packet_fence_radius_group_reply_content" "r_g_r_c",
        "roles"                      "r"
    WHERE
        "r"."id" = "r_g_r_c"."role_id"
;
-- }}}

GRANT CONNECT ON DATABASE atlas TO atlas_consistency_checker;
GRANT SELECT ON "users" TO "atlas_consistency_checker";

GRANT CONNECT ON DATABASE atlas TO systems_user;
GRANT CONNECT ON DATABASE atlas TO systems_shadow;
GRANT CONNECT ON DATABASE atlas TO web_user;

GRANT CONNECT ON DATABASE atlas TO generate_flat_files_user;

GRANT CONNECT ON DATABASE atlas TO radius_user;
GRANT CONNECT ON DATABASE atlas TO vpn_radius_user;
GRANT CONNECT ON DATABASE atlas TO netgear_radius_user;

GRANT CONNECT ON DATABASE "atlas" TO "rt_sirt_user";
GRANT INSERT, UPDATE ON "user_sanctions" TO "rt_sirt_user";
GRANT SELECT, USAGE ON "user_sanctions_id_seq" TO "rt_sirt_user";

GRANT SELECT (
    "username",
    "password",
    "id",
    "group_id",
    "comment",
    "home_directory",
    "legacy_home_directory",
    "uf_home_directory",
    "shell",
    "hostname",
    "host_id"
)
ON "systems_users" TO "systems_user";

GRANT SELECT (
    "username",
    "password",
    "id",
    "group_id",
    "comment",
    "home_directory",
    "legacy_home_directory",
    "uf_home_directory",
    "shell",
    "hostname",
    "host_id"
)
ON "systems_users" TO "generate_flat_files_user";

GRANT SELECT ON "systems_groups"       TO "systems_user";
GRANT SELECT ON "systems_users_groups" TO "systems_user";

GRANT SELECT ON "systems_groups"       TO "generate_flat_files_user";
GRANT SELECT ON "systems_users_groups" TO "generate_flat_files_user";

GRANT SELECT (
    "username",
    "encrypted_password",
    "date_of_last_password_change",
    "minimum_password_age",
    "maximum_password_age",
    "password_warning_period",
    "password_inactivity_period",
    "account_expiration_date",
    "reserved_field",
    "hostname"
)
ON "systems_users" TO "systems_shadow";

GRANT SELECT (
    "username",
    "encrypted_password",
    "date_of_last_password_change",
    "minimum_password_age",
    "maximum_password_age",
    "password_warning_period",
    "password_inactivity_period",
    "account_expiration_date",
    "reserved_field",
    "hostname"
)
ON "systems_users" TO "generate_flat_files_user";

GRANT SELECT ON "web_users" TO "web_user";

GRANT SELECT ON "radius_check"       TO "radius_user";
GRANT SELECT ON "radius_reply"       TO "radius_user";
GRANT SELECT ON "radius_group_check" TO "radius_user";
GRANT SELECT ON "radius_group_reply" TO "radius_user";
GRANT SELECT ON "radius_user_group"  TO "radius_user";
GRANT SELECT ON "radius_nas"         TO "radius_user";
GRANT SELECT ON "radius_nas_type"    TO "radius_user";

GRANT SELECT ON "radius_check"       TO "netgear_radius_user";
GRANT SELECT ON "radius_reply"       TO "netgear_radius_user";
GRANT SELECT ON "radius_group_check" TO "netgear_radius_user";
GRANT SELECT ON "radius_group_reply" TO "netgear_radius_user";
GRANT SELECT ON "radius_user_group"  TO "netgear_radius_user";
GRANT SELECT ON "radius_nas"         TO "netgear_radius_user";
GRANT SELECT ON "radius_nas_type"    TO "netgear_radius_user";

GRANT SELECT ON "radius_check"              TO "vpn_radius_user";
GRANT SELECT ON "vpn_radius_user_sanctions" TO "vpn_radius_user";
GRANT SELECT ON "radius_reply"              TO "vpn_radius_user";
GRANT SELECT ON "vpn_radius_group_check"    TO "vpn_radius_user";
GRANT SELECT ON "vpn_radius_group_reply"    TO "vpn_radius_user";
GRANT SELECT ON "vpn_radius_user_group"     TO "vpn_radius_user";

GRANT CONNECT ON DATABASE "atlas" TO "pf_user";
GRANT SELECT ON "packet_fence_radius_check" TO "pf_user";
GRANT SELECT ON "packet_fence_radius_reply"       TO "pf_user";
GRANT SELECT ON "packet_fence_radius_group_check" TO "pf_user";
GRANT SELECT ON "packet_fence_radius_group_reply" TO "pf_user";
GRANT SELECT ON "packet_fence_radius_user_group"  TO "pf_user";

GRANT CONNECT ON DATABASE "atlas" TO "locker_user";
GRANT SELECT (
    "given_name",
    "middle_initial",
    "surname",
    "university_id",
    "username"
)
ON "users" TO "locker_user";

GRANT CONNECT ON DATABASE "atlas" TO "webdrop_user";
GRANT SELECT (
    "username",
    "university_id",
    "freeform_name"
)
ON "users" TO "webdrop_user";

GRANT CONNECT ON DATABASE "atlas" TO "lsbe_scholarship_user";
GRANT SELECT (
    "username",
    "university_id",
    "freeform_name"
)
ON "users" TO "lsbe_scholarship_user";

GRANT CONNECT ON DATABASE "atlas" TO "rt_accounts_importer";
GRANT SELECT ON "users" TO "rt_accounts_importer";
GRANT SELECT ON "suppressed_demographics" TO "rt_accounts_importer";
GRANT SELECT ON "appointments" TO "rt_accounts_importer";

GRANT CONNECT ON DATABASE "atlas" TO "housing_user";
GRANT CONNECT ON DATABASE "atlas" TO "housing_test_user";
GRANT SELECT (
    "freeform_name",
    "address_peoplesoft_home",
    "address_peoplesoft_mailing",
    "surname",
    "given_name",
    "middle_initial",
    "phone_number_home",
    "birth_date",
    "username",
    "academic_class",
    "number_of_suppressed_fields",
    "gender",
    "admissions_data",
    "university_id"
)
ON "users" TO "housing_user";
GRANT SELECT (
    "freeform_name",
    "address_peoplesoft_home",
    "address_peoplesoft_mailing",
    "surname",
    "given_name",
    "middle_initial",
    "phone_number_home",
    "birth_date",
    "username",
    "academic_class",
    "number_of_suppressed_fields",
    "gender",
    "admissions_data",
    "university_id"
)
ON "users" TO "housing_test_user";

GRANT SELECT ON "hosts" TO "puppet_enc";
GRANT SELECT ON "classes" TO "puppet_enc";
GRANT SELECT ON "parameters" TO "puppet_enc";
GRANT SELECT ON "hosts_classes" TO "puppet_enc";


-- TODO This is Pg 8.4 specific view code
-- when upgrading atlas, please look at rewriting this view.
CREATE VIEW "ordered_classes" AS
    SELECT
        "ordered_classes"."host_id",
        array_to_string(array_agg("ordered_classes"."class"), E'\n') AS "classes"
    FROM
        (
            SELECT
                "h_c"."host_id",
                "c"."class"
            FROM       "classes"       "c"
            INNER JOIN "hosts_classes" "h_c" ON ("c"."id" = "h_c"."class_id")
            ORDER BY
                "h_c"."host_id" ASC,
                "c"."class"     ASC
        ) "ordered_classes"
    GROUP BY
        "ordered_classes"."host_id"
;
ALTER VIEW "ordered_classes" OWNER TO "atlas";


-- TODO This is Pg 8.4 specific view code
-- when upgrading atlas, please look at rewriting this view.
CREATE VIEW "ordered_parameters" AS
    SELECT
        "ordered_parameters"."host_id",
        array_to_string(array_agg("ordered_parameters"."name" || ' = ' || "ordered_parameters"."value"), E'\n') AS "parameters"
    FROM
        (
            SELECT
                "p"."host_id",
                "p"."name",
                "p"."value"
            FROM       "parameters" "p"
            ORDER BY
                "p"."host_id" ASC,
                "p"."name"    ASC
        ) "ordered_parameters"
    GROUP BY
        "ordered_parameters"."host_id"
;
ALTER VIEW "ordered_parameters" OWNER TO "atlas";

CREATE VIEW "hosts_puppet" AS
    SELECT
        "h"."id",
        "h"."hostname",
        "h"."environment",
        "o_c"."classes",
        "o_p"."parameters"
    FROM      "hosts"              "h"
    LEFT JOIN "ordered_classes"    "o_c" ON ("h"."id" = "o_c"."host_id")
    LEFT JOIN "ordered_parameters" "o_p" ON ("h"."id" = "o_p"."host_id")
    ORDER BY
        "h"."hostname" ASC
;
ALTER VIEW "hosts_puppet" OWNER TO "atlas";

-- TODO This is Pg 8.4 specific view code
-- when upgrading atlas, please look at rewriting this view.
CREATE VIEW "ordered_hosts" AS
    SELECT
        "ordered_hosts"."class_id",
        array_to_string(array_agg("ordered_hosts"."hostname"), E'\n') AS "hosts"
    FROM
        (
            SELECT
                "h_c"."class_id",
                "h"."hostname"
            FROM       "hosts"         "h"
            INNER JOIN "hosts_classes" "h_c" ON ("h"."id" = "h_c"."host_id")
            ORDER BY
                "h_c"."class_id" ASC,
                "h"."hostname"   ASC
        ) "ordered_hosts"
    GROUP BY
        "ordered_hosts"."class_id"
;
ALTER VIEW "ordered_hosts" OWNER TO "atlas";

CREATE VIEW "all_classes_with_hosts" AS
    SELECT
        "c"."class",
        "o_h"."hosts"
    FROM            "classes"       "c"
    LEFT OUTER JOIN "ordered_hosts" "o_h" ON ("c"."id" = "o_h"."class_id")
    ORDER BY
        "c"."class" ASC
;
ALTER VIEW "all_classes_with_hosts" OWNER TO "atlas";
